/*
 * Copyright 2003 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _AUTH_H_
#define _AUTH_H_

#include <openssl/hmac.h>
#include <openssl/blowfish.h>

struct auth_state {
	RSA *rsa_key;
	DH *dh;

	BIGNUM *p, *g, *dh_pub;

	u_char client_id[16];
	u_char digest[20];
	
	BF_KEY in;
	u_char iniv[8];
	HMAC_CTX authin;

	BF_KEY out;
	u_char outiv[8];
	HMAC_CTX authout;

	u_char *shared;
	size_t sharedlen;

	int flags;
	int step;

	struct io_duplex *dplx;
	struct event ev;
};

/* Client */
#define CAUTH_READRSAKEY		0
#define		CAUTH_RRK_GOTID		0x0001
#define		CAUTH_RRK_GOTP		0x0002
#define		CAUTH_RRK_GOTG		0x0004
#define CAUTH_READDHPUB			1
#define		CAUTH_RDP_GOTDHPUB	0x0001
#define		CAUTH_RDP_GOTSIG	0x0002

/* Server */
#define SAUTH_SENDRSAKEY	0
#define SAUTH_READDHPUB		1
#define SAUTH_READCLIENTID	2

void auth_free_state(struct auth_state *);
int auth_protocol_client(struct io_buffer *, struct auth_state *);
int auth_protocol_server(struct io_buffer *, struct auth_state *);

#endif /* !_AUTH_H_ */
