/*
 * Copyright 2003 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _COMMANDS_H_
#define _COMMANDS_H_

#define MAX_CMDLINE	256

struct client;

#define CCFLAG_STARTED		0x0001
#define CCFLAG_SCRIPT		0x0010		/* command is in a script */
#define CCFLAG_SCRIPTEND	0x0020		/* end of script */

struct client_cmd;

struct client_serialize_queue {
	TAILQ_HEAD(csq, client_cmd) csq_queue;
	int csq_max;
	int csq_current;
	int csq_pending;
};

struct client_cmd {
	TAILQ_ENTRY(client_cmd) cc_next;
	TAILQ_ENTRY(client_cmd) cc_csq_next;
	struct client_serialize_queue *csq_parent;
	struct client *cc_client;

	/* Sent to the client */
	char cc_command[MAX_CMDLINE];

	/* client callback and argument */
	int (*cc_cb)(int, struct client *, void *, char *);
	void *cc_cbarg;

	int cc_flags;
};

TAILQ_HEAD(cmdlist, client_cmd);

#define CMD_ERROR	"ERROR"
#define CMD_OK		"OK"
#define CMD_DONE	"DONE"
#define CMD_ABORT	"ABORT"
#define CMD_INIT	"INIT"
#define CMD_RUN		"RUN"
#define CMD_QUERY	"QUERY"

/* Unsollicted replies from the client */

#define CMD_STATUS		"STATUS"
#define CMD_INFO		"INFO"
#define		INFO_CHLDDONE	0
#define		INFO_CHLDDEAD	1
#define CMD_OUTPUT		"OUTPUT"

#define CMD_SZ(x)	(sizeof(x) - 1)

/* Prototypes */
int command_queue(struct cmdlist *, char *,
		  int (*)(int, struct client *, void *, char*), void *);
void command_dequeue(struct cmdlist *, struct client_cmd *);
void command_free(struct client *, struct client_cmd *);
int command_start(struct client *);
int command_reply(struct client *, char *, int);

struct client_cmd *command_script_start(struct cmdlist *);
void command_script_end(struct cmdlist *, struct client_cmd *);

struct client_serialize_queue *command_serialize_new(int);
void command_serialize_last(struct client_serialize_queue *, struct client *);

#endif /* !_COMMANDS_H_ */
