/*
 * Copyright 2003 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _SCHEDULE_H_
#define _SCHEDULE_H_

#include <bitstring.h>

#include "dataset.h"

#define SLFLAGS_CLIENTGONE	0x0001
#define SLFLAGS_DIED		0x0002

#define SLOT_TIMEOUT		(1*60*60)
#define SLOT_WAIT		3

struct slot {
	TAILQ_ENTRY(slot) sl_next;

	struct schedule *sl_sd;

	int sl_slot;
	time_t sl_when;

	u_char sl_clid[16];
	char *sl_clname;
	u_short sl_clport;

	int sl_flags;

	struct event sl_timeout;
	struct event sl_wait;
};

#define SDFLAGS_AUTOMATIC	0x0001
#define SDFLAGS_LOCKED		0x0002

struct schedule {
	TAILQ_ENTRY(schedule) sd_next;

	TAILQ_HEAD(slotlist, slot) sd_slqueue;

	int sd_start;	/* Parameter start */
	int sd_end;		/* Parameter end */

	u_int sd_assigned;	/* Currently assigned */
	u_int sd_orphaned;	/* Currently assigned, but lost */
	u_int sd_done;	/* Done */
	u_int sd_left;	/* Work left */

	time_t	sd_timecount;	/* Complete time for each slot */
	time_t	sd_timemin;	/* Minimum time */
	time_t	sd_timemax;	/* Maximum time */
	time_t	sd_timestart;	/* Time this job started */
	time_t  sd_timeend;
	
	bitstr_t *sd_bitmap;
	size_t sd_nbits;

	char *sd_id;

	char *sd_cmd;

	int sd_flags;

	/* Files that belong to this schedule */
	struct dataset sd_files[MAX_DATASETS+1];
	int sd_nfiles;

	/* Output files */
	int sd_stdout;
	int sd_stderr;
};

struct slot *slot_new(struct schedule *);
void slot_free(struct slot *);
struct slot *slot_find_id(struct schedule *, u_char *);
struct slot *slot_find_nr(struct schedule *, int);
void slot_result(struct schedule *, struct slot *, int);
void slot_lostclient(int, short, void *);

void schedule_init(void);
struct schedule *schedule_new(char *);
void schedule_free(struct schedule *);
struct schedule *schedule_find(char *);

char *schedule_status(struct schedule *, int);
void schedule_update(struct schedule *);

char *schedule_expand(struct schedule *, char *, int);

struct client;
void schedule_clientdie(struct client *);
void schedule_clientconnect(struct client *, char *);
void schedule_slotdone(struct client *, char *, int);
int schedule_assign(struct schedule *, struct client *, int *);
void schedule_returnslot(struct schedule *, int);
void schedule_reclaim(struct schedule *sd, struct io_buffer *);

void schedule_list(struct io_buffer *);

#define SD_HAS_RANGE(x)	((x)->sd_start != -1 && (x)->sd_end != -1)

#endif /* !_SCHEDULE_H_ */
