/*
 * Copyright 2003 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _SERVER_H_
#define _SERVER_H_

#include "commands.h"

#define CLIENT_CMDINPROGRESS	0x0001
#define CLIENT_ONCMDLIST	0x0002
#define CLIENT_CMDREADY		0x0004
#define CLIENT_DEAD		0x0008
#define CLIENT_CMDSTAGED	0x0010

#define CLIENT_AUTHDONE		0x1000

#define CLIENT_WRITETIMEOUT	(3*60)

struct client {
	TAILQ_ENTRY(client) client_next;
	TAILQ_ENTRY(client) cmd_next;

	struct cmdlist client_cmdqueue;

	char *name;
	u_short port;
	time_t when;

	char status[128];

	struct io_filter *src;	/* Data from here is decrypted */
	struct io_filter *dst;	/* Data that goes here will be encrypted */

	struct io_filter *auth_filter;

	int tap;
	int flags;

	struct io_duplex *dplx;
	struct auth_state *auth;

	struct event ev;
};

TAILQ_HEAD(clientlist, client);

void client_init(void);
void client_free(struct client *);
struct client *client_new(char *, u_short);
struct client *client_find_id(u_char *);

int client_onqueue(struct io_buffer *);

void client_list(struct io_buffer *, char *);
void client_kill(struct io_buffer *, char *);

struct schedule;
void client_run(struct io_buffer *, struct schedule *);
void client_run_cmd(struct io_buffer *, char *, char *);
void client_run_one(struct client *, struct schedule *);

void client_settimeout(struct client *);

#define CMD_TAP		0x0001

int filter_clientresponse(void *, struct io_filter *, struct io_buffer *, int);
int client_select(struct io_buffer *, char *);

void client_allready(void);
void client_dead(struct io_obj *, void *);

#endif /* !_SERVER_H_ */
